package cmdVM

import (
    "fmt"
    "strconv"
    "nexus-common/params"
    u "nexus-client/utils"
    g "nexus-client/globals"
    "nexus-common/vm"
    "github.com/google/uuid"
)

type Create struct {
    Name string
}

func (cmd *Create)GetName() string {
    return cmd.Name
}

func (cmd *Create)GetDesc() []string {
    return []string{
        "Creates one or more VMs.",
        "Requires VM_CREATE user capability."}
}

func (cmd *Create)PrintUsage() {
    for _, desc := range cmd.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: "+cmd.GetName()+" name cpus ram nic usb templateID [count|file.csv]")
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    const usage string = `name        Name of the VM to create.
cpus        Number of CPUs, between 1 and 32.
ram         Amount of RAM in MB, between 128 and 32768.
nic         Network interface, either "none" (no network) or "user" (network access).
usb         List of USB devices exposed in the VM; either "none" or a list of comma separated
            vendorID:productID (4-digit hex number), each specifying a USB device; example
            exposing two USB devices: 1fc9:001d,067b:2303
templateID  ID of the template to VM will be based on.
count       Number of VMs to create (if not specified, one is created).
            If specified and > 1, each VM's name is postfixed with [n],
            where n ranges from 1..count.
file.csv    1-column CSV file defining the students names.
            Each VM's name is postfixed with each entry in the CSV file.`
    u.PrintlnErr(usage)
}

func (cmd *Create)Run(args []string) int {
    client := g.GetInstance().Client
    host := g.GetInstance().Host

    argc := len(args)
    if argc < 6 || argc > 7  {
        cmd.PrintUsage()
        return 1
    }

    name := args[0]
    ncpus, err := strconv.Atoi(args[1])
    if err != nil {
        u.PrintlnErr("Invalid number of CPU(s)")
        return 1
    }
    ram, err := strconv.Atoi(args[2])
    if err != nil {
        u.PrintlnErr("Invalid amount of RAM")
        return 1
    }
    nic := vm.NicType(args[3])
    usbDevs := u.Str2UsbDevices(args[4])
    templateID, err := uuid.Parse(args[5])
    if err != nil {
        u.PrintlnErr(err)
    }

    count := 1
    var csvEntries []string = nil

    // If there is a 6th argument, checks whether it's a number or a CSV file.
    if argc == 7 {
        count, err = strconv.Atoi(args[6])
        if err != nil {
            // It's not a number, we assume it's a CSV file and parse it.
            csvFile := args[6]
            csvEntries, err = u.ReadCSVColumn(csvFile, 0)
            if err != nil {
                u.PrintlnErr(err.Error())
                return 1
            }
            count = len(csvEntries)
        } else {
            if count < 1 {
                u.PrintlnErr("count must be > 0")
                return 1
            }
        }
    }

    // At this point:
    // if csvFile == "" -> we must create count VMs
    // otherwise -> we parse the CSV to know how many VMs to create

    vmArgs := &params.VMCreate {
        Name: name,
        Cpus: ncpus,
        Ram: ram,
        Nic: nic,
        UsbDevs: usbDevs,
        TemplateID: templateID,
    }

    statusCode := 0
    digits := len(strconv.Itoa(count))

    for i := 1; i <= count; i++ {
        if csvEntries == nil {
            if count > 1 {
                numberPadded := fmt.Sprintf("%0"+strconv.Itoa(digits)+"d", i)
                vmArgs.Name = name+" "+numberPadded
            }
        } else {
            vmArgs.Name = name+" "+csvEntries[i-1]
        }

        resp, err := client.R().SetBody(vmArgs).Post(host+"/vms")
        if err != nil {
            u.PrintlnErr("Failed creating VM \""+vmArgs.Name+"\": "+err.Error())
            statusCode = 1
        } else {
            if resp.IsSuccess() {
                vm, err := deserializeVM(resp)
                if err != nil {
                    u.PrintlnErr("Failed retrieving server's response: "+err.Error())
                    statusCode = 1
                }
                u.Println("Created VM \""+vm.Name+"\" | "+vm.ID.String())
            } else {
                u.PrintlnErr("Failed creating VM \""+vmArgs.Name+"\": "+resp.Status()+": "+resp.String())
                statusCode = 1
            }
        }
    }

    return statusCode
}
